/*
 * Copyright (c) 2010 Paul de Weerd <weerd@weirdnet.nl>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/param.h>

#include <err.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#ifndef __dead
#define __dead
#endif

__dead void usage(void);

int
main(int argc, char *argv[]) {
	u_int32_t bnd = -1, cnt = 1;
	int quiet = 0, userbound = 0, c;
	const char *errstr;

	while ((c = getopt(argc, argv, "b:c:q")) != -1)
		switch (c) {
		case 'b':
			bnd = (u_int32_t)strtonum(optarg, 1, UINT_MAX, &errstr);
			if (errstr)
				errx(1, "bound is %s: %s", errstr, optarg);
			userbound = 1;
			break;
		case 'c':
			cnt = (u_int32_t)strtonum(optarg, 0, UINT_MAX, &errstr);
			if (errstr)
				errx(1, "count is %s: %s", errstr, optarg);
			break;
		case 'q':
			quiet = 1;
			break;
		default:
			usage();
		}

	argc -= optind;
	argv += optind;

	if (cnt != 1 && quiet == 1)
		errx(1, "quiet mode needs -c 1");

	if (argc > 1)
		usage();

	if (argc == 1) {
		if (userbound == 1) {
			usage();
		} else {
			bnd = (u_int32_t)strtonum(*argv, 1, UINT_MAX, &errstr);
			if (errstr)
				errx(1, "bound is %s: %s", errstr, *argv);
			userbound = 1;
		}
	}

	if (quiet == 1) {
		if (bnd > 256)
			bnd = 256;
		return ((int)arc4random_uniform(bnd));
	} else {
		if (cnt == 0) {
			for (;;)
				if (userbound == 1) {
					printf("%u\n", arc4random_uniform(bnd));
				} else {
					printf("%u\n", arc4random());
				}
		} else {
			for(;cnt != 0; cnt--)
				if (userbound == 1) {
					printf("%u\n", arc4random_uniform(bnd));
				} else {
					printf("%u\n", arc4random());
				}
			return (0);
		}
	}
}

__dead void
usage(void)
{
	extern char *__progname;

	printf("usage: %s [-q | -c count] [[-b] bound]\n", __progname);
	exit(1);
}
